#!/usr/bin/python

""" userdefined script which builds a site """

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# Copyright 2004 Dries Verachtert

import sys, string, os, urllib,re ,shutil
from datetime import datetime
sys.path.append("/usr/share/pydar2/pydar")
sys.path.append("pydar")
sys.path.append(".")
from log4py import Logger
from config import Config
import storagefactory

# currently this builds the site at dries.ulyssis.org

def RpmforgeSite(specRepo):
    print "start, test"
    so = SiteObject(specRepo)
    so.generate()
    
class DistroArchObject:
    def __init__(self,shortName,displayName, rpmsUrl, suffixes, fullName, additionalPaths):
        self.shortName = shortName
        self.displayName = displayName
        self.rpmsUrl = rpmsUrl
        self.rpms = []
        self.srpms = []
        self.suffixes = suffixes
        self.fullName = fullName
        self.additionalPaths = additionalPaths
    
class SiteObject:
    def __init__(self,specRepo):
        self.__cat = Logger().get_instance(self)
        self.__cat.debug("start")
        # this is a hash which contains values which can be used in templates
        # for example FC3-I386-DRIES-RPMS -> 533     (the number of rpms)
        #                       LASTUPDATE -> 4 jan 2010 5:07
        self.__siteValues = {}
        self.__specRepo = specRepo
        self.__fullcontentstemplate = None
        self.__siterootdir =  "/tmp/packages/"
        self.__storage = storagefactory.StorageFactory.getStorage()
        self.__rpmlists = []
        addPaths = ['redhat/el4/en/i386']
        self.__rpmlists.append(DistroArchObject("el4-i386-dries", "el4 i386 rpms","http://ftp.belnet.be/packages/dries.ulyssis.org/redhat/el4/en/i386/RPMS.dries/", ["el4.rf"], "Red Hat Enterprise Linux 4 for i386", addPaths))
        addPaths = ['redhat/el3/en/i386']
        self.__rpmlists.append(DistroArchObject("el3-i386-dries", "el3 i386 rpms","http://ftp.belnet.be/packages/dries.ulyssis.org/redhat/el3/en/i386/RPMS.dries/", ["el3.rf"], "Red Hat Enterprise Linux 3 for i386", addPaths))
        addPaths = ['fedora/fc3/i386']
        self.__rpmlists.append(DistroArchObject("fc3-i386-dries", "fc3 i386 rpms", "http://ftp.belnet.be/packages/dries.ulyssis.org/fedora/fc3/i386/RPMS.dries/", ["fc3.rf", "fc3.dries"], "Fedora Core 3 for i386", addPaths))
        addPaths = ['fedora/fc2/i386']
        self.__rpmlists.append(DistroArchObject("fc2-i386-dries", "fc2 i386 rpms", "http://ftp.belnet.be/packages/dries.ulyssis.org/fedora/fc2/i386/RPMS.dries/", ["fc2.rf", "fc2.dries"], "Fedora Core 2 for i386", addPaths))
        addPaths = ['fedora/fc1/i386']
        self.__rpmlists.append(DistroArchObject("fc1-i386-dries", "fc1 i386 rpms", "http://ftp.belnet.be/packages/dries.ulyssis.org/fedora/fc1/i386/RPMS.dries/", ["fc1.rf", "fc1.dries"], "Fedora Core 1 for i386", addPaths))
        addPaths = ['fedora/fc4/i386']
        self.__rpmlists.append(DistroArchObject("fc4-i386-dries", "fc4 i386 rpms", "http://ftp.belnet.be/packages/dries.ulyssis.org/fedora/fc4/i386/RPMS.dries/", ["fc4.rf", "fc4.dries"], "Fedora Core 4 for i386", addPaths))
        addPaths = ['fedora/fc4/x86_64']
        self.__rpmlists.append(DistroArchObject("fc4-x86_64-dries", "fc4 x86_64 rpms", "http://ftp.belnet.be/packages/dries.ulyssis.org/fedora/fc4/x86_64/RPMS.dries/", ["fc4.rf", "fc4.dries"], "Fedora Core 4 for x86_64", addPaths))
        addPaths = ['aurora/1.92/sparc']
        self.__rpmlists.append(DistroArchObject("au1.92-sparc-dries", "au1.92 sparc rpms", "http://ftp.belnet.be/packages/dries.ulyssis.org/aurora/1.92/sparc/RPMS.dries/", ["au1.92.rf", "au1.92.dries"], "Aurora 1.92 for sparc", addPaths))
        addPaths = ['aurora/1.91/sparc']
        self.__rpmlists.append(DistroArchObject("au1.91-sparc-dries", "au1.91 sparc rpms", "http://ftp.belnet.be/packages/dries.ulyssis.org/aurora/1.91/sparc/RPMS.dries/", ["au1.91.rf", "au1.91.dries"],"Aurora 1.91 for sparc", addPaths))
        self.__rpmNameToSpecSubDirName = {}
        self.__completeRpmsLists()
        self.__completeSiteValues()
        
    # counters is a hash which contains interesting values which can be used on pages
    def __completeSiteValues(self):
        # it should already contain
        # EL4-I386-DRIES-RPMS, EL4-I386-DRIES-SRPMS, ...
        self.__siteValues["NUMBERSPECFILES"] = len(self.__specRepo.getSpecRepositorySpecFiles())
        self.__siteValues["LASTUPDATE"] = str(datetime.today())
        totrpms = 0
        totsrpms = 0
        for d in self.__rpmlists:
            totrpms = totrpms + len(d.rpms)
            totsrpms = totsrpms + len(d.srpms)
        self.__siteValues["TOTALRPMS"] = totrpms
        self.__siteValues["TOTALSRPMS"] = totsrpms
        
    def __completeRpmsLists(self):
        # list of packagenames
        packagenames = self.__getPackageNameListSortedByLength()
        # make a hash packagename -> specrepospecfile
        nametospecrepospecfile = {}
        for srsf in self.__specRepo.getSpecRepositorySpecFiles():
            nametospecrepospecfile[srsf.getSubDir()] = srsf
        #firstre = re.compile('<A HREF=".*\.rpm<\/A>')
        secondre = re.compile('<A HREF="(.*\.rpm)">.*\.rpm</A> *([0-9]*)-([A-Za-z]*)-([0-9]*) ')
        for listje in self.__rpmlists:
            # ok get the list of rpms and srpms of this list
            f = urllib.urlopen(listje.rpmsUrl)
            #print f.info()
            print  "start, " + listje.rpmsUrl
            lijn = f.readline()
            while lijn != "":
                #print l
                mo = secondre.search(lijn)
                if mo != None:
                    rpm = mo.group(1)
                    listje.rpms.append(rpm)
                    i = 0
                    while ((i < len(packagenames)) and (not rpm.startswith(packagenames[i]))):
                        i = i + 1
                        #print i, len(packagenames)
                    if i<len(packagenames):
                        #print "match: " + rpm + ", package=" + packagenames[i]
                        nametospecrepospecfile[packagenames[i]].rpmfilenames.append(listje.rpmsUrl + rpm)
                        self.__rpmNameToSpecSubDirName[rpm] = packagenames[i]
                lijn = f.readline()
            f.close()
            self.__siteValues[listje.shortName.upper() + "-RPMS"] = len(listje.rpms)
            
        # the same for the sprms
        for listje in self.__rpmlists:
            # ok get the list of rpms and srpms of this list
            f = urllib.urlopen(listje.rpmsUrl.replace("RPMS","SRPMS"))
            #print f.info()
            print "start, " + listje.rpmsUrl.replace("RPMS","SRPMS")
            lijn = f.readline()
            while lijn != "":
                #print l
                mo = secondre.search(lijn)
                if mo != None:
                    rpm = mo.group(1)
                    listje.srpms.append(rpm)
                    i = 0
                    while ((i < len(packagenames)) and (not rpm.startswith(packagenames[i]))):
                        i = i + 1
                        #print i, len(packagenames)
                    if i<len(packagenames):
                        #print "match: " + rpm + ", package=" + packagenames[i]
                        nametospecrepospecfile[packagenames[i]].srpmfilenames.append(listje.rpmsUrl.replace("RPMS","SRPMS") + rpm)
                        self.__rpmNameToSpecSubDirName[rpm] = packagenames[i]
                lijn = f.readline()
            f.close()
            self.__siteValues[listje.shortName.upper() + "-SRPMS"] = len(listje.srpms)
        
    def __getPackageNameListSortedByLength(self):
        retval = []
        for srsf in self.__specRepo.getSpecRepositorySpecFiles():
            name = srsf.getSubDir()
            retval.append(name)
        retval.sort(lambda x, y : len(y)-len(x))
        return retval
        
    def generate(self):
        # generate the content
        print "start, static pages"
        self.generateIndexPage()
        self.generateMirrorsPage()
        self.generatePackagesPage()
        self.generateGPGKeyPage()
        self.clientConfigPage()
        self.pydar2Pages()
        self.setGPGKey()
        print "start, specrepospecfile pages"
        for i in self.__specRepo.getSpecRepositorySpecFiles():
            self.generateOnePackagePage(i)
        print "start, distropages"
        for i in self.__rpmlists:
            self.generateDistroArchPageRpm(i)
            self.generateDistroArchPageSrpm(i)
        # also create some directory indexes
        dirs = ["fedora", "redhat", "aurora"]
        for i in dirs:
            self.generateDirectoryView(i)
        # add some symbolic links
        self.createDirHierarchy("fedora/core")
        self.createDirHierarchy("fedora/linux")
        self.createSymbolicLink("fedora/core/1", "../fc1")
        self.createSymbolicLink("fedora/core/2", "../fc2")
        self.createSymbolicLink("fedora/core/3", "../fc3")
        self.createSymbolicLink("fedora/core/4", "../fc4")
        self.createSymbolicLink("fedora/linux/1", "../fc1")
        self.createSymbolicLink("fedora/linux/2", "../fc2")
        self.createSymbolicLink("fedora/linux/3", "../fc3")
        self.createSymbolicLink("fedora/linux/4", "../fc4")
        self.generateDirectoryView("fedora")
        self.generateDirectoryView("fedora/core")
        self.generateDirectoryView("fedora/linux")
        self.generateDirectoryView("redhat")
        self.generateDirectoryView("aurora")
        
    def generateDirectoryView(self,path):
        contents = self.__getFileContents("dries/directoryview-part.html")
        self.generateFullPage(contents,[],path + "/index.php", "Index of /rpm/" + path + " - Dries RPM Repository")
        
            
    def pydar2Pages(self):
        contents = self.__getFileContents("dries/pydar2/index-part.html")
        self.generateFullPage(contents,[],"pydar2/index.html", "Pydar2 rpm buildserver: Dries RPM Repository")
        contents = self.__getFileContents("dries/pydar2/install-part.html")
        self.generateFullPage(contents,[],"pydar2/install.html", "Installation: Pydar2 rpm buildserver: Dries RPM Repository")
        contents = self.__getFileContents("dries/pydar2/install-slave-part.html")
        self.generateFullPage(contents,[],"pydar2/install-slave.html", "Installation: Pydar2 rpm buildserver: Dries RPM Repository")
        contents = self.__getFileContents("dries/pydar2/terminology-part.html")
        self.generateFullPage(contents,[],"pydar2/terminology.html", "Terminology: Pydar2 rpm buildserver: Dries RPM Repository")
        contents = self.__getFileContents("dries/pydar2/planning-part.html")
        self.generateFullPage(contents,[],"pydar2/planning.html", "Planning: Pydar2 rpm buildserver: Dries RPM Repository")
        contents = self.__getFileContents("dries/pydar2/classdiagram-part.html")
        self.generateFullPage(contents,[],"pydar2/classdiagram.html", "Class diagram: Pydar2 rpm buildserver: Dries RPM Repository")
        contents = self.__getFileContents("dries/pydar2/needed-part.html")
        self.generateFullPage(contents,[],"pydar2/needed.html", "Todo: Pydar2 rpm buildserver: Dries RPM Repository")
        shutil.copy("dries/pydar2/classdiagram.png",self.__siterootdir + "/pydar2/")
        shutil.copy("dries/pydar2/classdiagram.xmi",self.__siterootdir + "/pydar2/")
            
    def clientConfigPage(self):
        contents = self.__getFileContents("dries/clientconfig-part.html")
        self.generateFullPage(contents,[],"clientconfig.html", "Smart, Yum or Apt configuration: Dries RPM Repository")
            
    def generateIndexPage(self):
        contents = self.__getFileContents("dries/index-part.html")
        self.generateFullPage(contents,[],"index.html", "Dries RPM Repository")
        
    def generateGPGKeyPage(self):
        contents = self.__getFileContents("dries/RPM-GPG-KEY.dries-part.html")
        self.generateFullPage(contents,[],"RPM-GPG-KEY.dries.html", "GPG Key: Dries RPM Repository")
     
    def setGPGKey(self):
        shutil.copy("dries/RPM-GPG-KEY.dries.txt",self.__siterootdir)
        
    def generateMirrorsPage(self):
        contents = self.__getFileContents("dries/mirrors-part.html")
        self.generateFullPage(contents,[],"mirrors.html", "Mirrors: Dries RPM Repository")
        
        
    def generateDistroArchPageRpm(self,distroArch):
        contents = ""
        contents = "<h2>Rpms for " + distroArch.fullName + " (" + distroArch.displayName + ")</h2>"
        for r in distroArch.rpms:
            linkToPackage = ""
            if r in self.__rpmNameToSpecSubDirName:
                linkToPackage = "&nbsp;-&nbsp<a href=\"/rpm/packages/" + self.__rpmNameToSpecSubDirName[r] + "/info.html\">" + self.__rpmNameToSpecSubDirName[r] + "</a>"
            contents = contents + "<a href=\"" + distroArch.rpmsUrl + r + "\">" + r + "</a>" + linkToPackage + "<br />"
        self.generateFullPage(contents, [distroArch.displayName], distroArch.shortName + "-rpms.html",distroArch.fullName + " (" + distroArch.displayName + "): Dries RPM Repository")
        # also generate a page in redhat/el4/i386/ for example
        for path in distroArch.additionalPaths:
            # 1 create a .htaccess file which links release.* files in this DIR
            # 2 create a dir DIR/dries , DIR/dries/headers , DIR/base, DIR/RPMS.dries, DIR/SRPMS.dries DIR/RPMS.dries/repodata DIR/SRPMS.dries/repodata
            # 3 create a symbolic link: DIR/dries/RPMS -> ../RPMS.dries
            # 4 create a symbolic link: DIR/dries/SRPMS -> ../SRPMS.dries
            # 5 create a DIR/dries/headers/.htaccess which redirects all .hdr files and the header.info file
            # 6 create a DIR/base/.htaccess  which redirects all pkglist.* release.* and srclist.* files
            # 7 create a DIR/RPMS.dries/.htaccess which redirects all  .*\.rpm files
            # 8 create a DIR/RPMS.dries/repodata/.htaccess which redirects all filelist* other* primary* repomd* files
            # 9 create a DIR/SRPMS.dries/.htaccess which redirects all .*\.rpm files
            # 10 create a DIR/SRPMS.dries/repodata/.htaccess which redirects all filelist* other* primary* repomd* files
            # 11 create dirindexes in DIR, DIR/dries, DIR/dries/headers, DIR/base
            # 1
            htcontents = "RewriteEngine on\n" + 'RewriteRule ^(release.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/$1' + "\n"
            self.createFile(os.path.join(path,".htaccess"), htcontents)
            # 2
            self.createDirHierarchy(os.path.join(path,"dries/headers"))
            self.createDirHierarchy(os.path.join(path,"base"))
            self.createDirHierarchy(os.path.join(path,"RPMS.dries/repodata"))
            self.createDirHierarchy(os.path.join(path,"SRPMS.dries/repodata"))
            # 3
            self.createSymbolicLink(os.path.join(path,"dries/RPMS"), "../RPMS.dries")
            # 4
            self.createSymbolicLink(os.path.join(path,"dries/SRPMS"), "../SRPMS.dries")
            # 5
            htcontents = "RewriteEngine on\n" + 'RewriteRule ^(.*hdr)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/dries/headers/$1' + "\n"
            htcontents = htcontents + 'RewriteRule ^(header.info)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/dries/headers/$1' + "\n"
            self.createFile(os.path.join(path,"dries/headers/.htaccess"), htcontents)
            # 6
            htcontents = "RewriteEngine on\n" + 'RewriteRule ^(pkglist.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/base/$1' + "\n"
            htcontents = htcontents + 'RewriteRule ^(release.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/base/$1' + "\n"
            htcontents = htcontents + 'RewriteRule ^(srclist.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/base/$1' + "\n"
            self.createFile(os.path.join(path,"base/.htaccess"), htcontents)
            # 7
            htcontents = "RewriteEngine on\n" + 'RewriteRule ^(.*.rpm)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/RPMS.dries/$1' + "\n"
            self.createFile(os.path.join(path,"RPMS.dries/.htaccess"), htcontents)
            # 8
            htcontents = "RewriteEngine on\n" + 'RewriteRule ^(filelist.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/RPMS.dries/repodata/$1' + "\n"
            htcontents = htcontents + 'RewriteRule ^(other.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/RPMS.dries/repodata/$1' + "\n"
            htcontents = htcontents + 'RewriteRule ^(primary.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/RPMS.dries/repodata/$1' + "\n"
            htcontents = htcontents + 'RewriteRule ^(repomd.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/RPMS.dries/repodata/$1' + "\n"
            self.createFile(os.path.join(path,"RPMS.dries/repodata/.htaccess"), htcontents)
            # 9
            htcontents = "RewriteEngine on\n" + 'RewriteRule ^(.*.rpm)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/SRPMS.dries/$1' + "\n"
            self.createFile(os.path.join(path,"SRPMS.dries/.htaccess"), htcontents)
            # 10
            htcontents = "RewriteEngine on\n" + 'RewriteRule ^(filelist.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/SRPMS.dries/repodata/$1' + "\n"
            htcontents = htcontents + 'RewriteRule ^(other.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/SRPMS.dries/repodata/$1' + "\n"
            htcontents = htcontents + 'RewriteRule ^(primary.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/SRPMS.dries/repodata/$1' + "\n"
            htcontents = htcontents + 'RewriteRule ^(repomd.*)$ http://ftp.belnet.be/packages/dries.ulyssis.org/' + path + '/SRPMS.dries/repodata/$1' + "\n"
            self.createFile(os.path.join(path,"SRPMS.dries/repodata/.htaccess"), htcontents)
            # 11
            self.generateDirectoryView(path)
            self.generateDirectoryView(path + "/dries")
            self.generateDirectoryView(path + "/dries/headers")
            self.generateDirectoryView(path + "/base")
            
            contents = "This page now contains links to the rpms on one of the mirrors because the page is generating way too much traffic for this server.<br /><br />" + contents
            self.generateFullPage(contents, [distroArch.displayName], path + "/RPMS.dries/index.html",distroArch.fullName + " (" + distroArch.displayName + "): Dries RPM Repository")
    
    def createSymbolicLink(self, filename, target):
        (tempdir, fname) = os.path.split(filename)
        dir = os.path.join(self.__siterootdir, tempdir)
        #print dir
        errors = 0
        try:
            os.makedirs(dir)
        except:
            errors = errors + 1
        try:
            os.symlink(target, os.path.join(dir,fname))
        except:
            errors = errors + 1
            
    def createFile(self, filename, contents):
        # first make sure the directory exists
        (tempdir, fname) = os.path.split(filename)
        dir = os.path.join(self.__siterootdir, tempdir)
        #print dir
        errors = 0
        try:
            os.makedirs(dir)
        except:
            errors = errors + 1
        f = open(os.path.join(dir,fname), "w")
        f.write(contents)
        f.close()
        
        
            
    def createDirHierarchy(self, path):
        dir = os.path.join(self.__siterootdir, path)
        errors = 0
        try:
            os.makedirs(dir)
        except:
            errors = errors + 1
            
    def generateDistroArchPageSrpm(self,distroArch):
        contents = ""
        url = distroArch.rpmsUrl.replace("RPMS","SRPMS")
        contents = "<h2>Srpms for " + distroArch.fullName + " (" + distroArch.displayName + ")</h2>"
        for r in distroArch.rpms:
            linkToPackage = ""
            if r in self.__rpmNameToSpecSubDirName:
                linkToPackage = "&nbsp;-&nbsp<a href=\"/rpm/packages/" + self.__rpmNameToSpecSubDirName[r] + "/info.html\">" + self.__rpmNameToSpecSubDirName[r] + "</a>"
            contents = contents + "<a href=\"" + url + r + "\">" + r + "</a>" + linkToPackage + "<br />"
        self.generateFullPage(contents, [distroArch.displayName], distroArch.shortName + "-srpms.html",distroArch.fullName + " (" + distroArch.displayName + "): Dries RPM Repository")
        for path in distroArch.additionalPaths:
            contents = "This page now contains links to the source rpms on one of the mirrors because the page is generating way too much traffic for this server.<br /><br />" + contents
            self.generateFullPage(contents, [distroArch.displayName], path + "/SRPMS.dries/index.html",distroArch.fullName + " (" + distroArch.displayName + "): Dries RPM Repository")
            
    def generatePackagesPage(self):
        contents = ""
        for i in self.__specRepo.getSpecRepositorySpecFiles():
            contents = contents + "<a href=\"packages/" + i.getSubDir() + "/info.html\">" + i.getSubDir() + "</a><br />"
        print contents
        self.generateFullPage(contents, ["Packages"], "packages.html","List of rpm packages: Dries RPM Repository")
        
    def fillFullContentsTemplate(self):
        self.__fullcontentstemplate = self.__getFileContents("dries/fullcontentstemplate.txt")
    
    def __getFileContents(self,fileName):
        tmp  = ""
        f = open(fileName)
        l = f.readline()
        while l != "":
            tmp = tmp + l
            l = f.readline()
        f.close()
        return tmp
        
    def __getMenu(self,levels):
        menu = ""
        menu = menu + "<strong>General</strong><br />"
        menu = menu + ""
        menu = menu + "<a href=\"/rpm/\">Start</a><br />"
        menu = menu + "<a href=\"/rpm/clientconfig.html\">APT/YUM/Smart config</a><br />"
        menu = menu + "<a href=\"/rpm/packages.html\">List of packages</a><br />"
        menu = menu + "<a href=\"/rpm/RPM-GPG-KEY.dries.html\">GPG key</a><br />"
        menu = menu + "<a href=\"/rpm/mirrors.html\">mirrors</a><br />"
        menu = menu + "<a href=\"/rpm/pydar2/index.html\">Pydar2</a><br />"
        menu = menu + "<strong>Rpms:</strong><br />"
        for k in self.__rpmlists:
            menu = menu + "<a href=\"/rpm/" + k.shortName + "-rpms.html\">" + k.displayName + "</a><br />"
        menu = menu + "<strong>Srpms:</strong><br />"
        for k in self.__rpmlists:
            menu = menu + "<a href=\"/rpm/" + k.shortName + "-srpms.html\">" + k.displayName.replace("rpms","srpms") + "</a><br />"
        menu = menu + "<strong>Thanks:</strong><br />"
        menu = menu + "Buildsystem hosted at <a href=\"http://www.ithomi.com/\">ithomi</a><br />"
        menu = menu + "Static site hosted at <a href=\"http://www.ulyssis.org/\">ULYSSIS</a><br />"
        menu = menu + "Primary mirrors hosted at <a href=\"http://www.belnet.be/\">BELNET</a>, <a href=\"http://www.hea.net/\">HEAnet</a>, <a href=\"http://3ti.ericom.be/\">3TI</a><br />"
        return menu
        
        
    def generateFullPage(self, contents, levels, filename, title):
        if self.__fullcontentstemplate == None:
            self.fillFullContentsTemplate()
        full = self.__fullcontentstemplate
        full = string.replace(full,"CONTENTS",contents)
        full = string.replace(full,"PAGETITLE", title)
        full = string.replace(full,"TITLEAREA", "<h1>" + title + "</h1>")
        full = string.replace(full,"BOTTOM", "Send reactions to: <a href=\"mailto:dries@ulyssis.org\">Dries Verachtert &lt;dries@ulyssis.org&gt;</a>") #+ "<br />Last update on " + self.__siteValues["LASTUPDATE"])
        full = string.replace(full,"MENU",self.__getMenu(levels))
        for k in self.__siteValues.keys():
            full = string.replace(full,k,str(self.__siteValues[k]))
        
        # first make sure the directory exists
        (tempdir, fname) = os.path.split(filename)
        dir = os.path.join(self.__siterootdir, tempdir)
        #print dir
        errors = 0
        try:
            os.makedirs(dir)
        except:
            errors = errors + 1
        f = open(os.path.join(dir,fname), "w")
        f.write(full)
        f.close()
        
        
    def generateOnePackagePage(self, specRepoSpecFile):
        versions = self.__storage.getVersionIdsOfSpecRepoSpecFile(specRepoSpecFile)
        arr = self.__storage.getTagsOfSpecFileVersion(versions[0])
        self.generateOnePackagePageFirstPage(specRepoSpecFile, versions, arr)
        self.generateOnePackagePageSpecPage(specRepoSpecFile)
        for v in versions:
            self.generateOnePackagePageVersionPage(specRepoSpecFile, v)
    
    def generateOnePackagePageVersionPage(self,specRepoSpecFile, versionId):
        shortName = specRepoSpecFile.getSubDir()
        contents = ""
        contents = contents + "<a href=\"info.html\">Back to the " + shortName + " rpms</a><br />"
        
        contents = contents + "<span class=\"subtitle\">versionId " + str(versionId) + ":</span><br />"
        contents = contents + "<table>" # <tr><th>Distro ids</th><th>Tag</th><th>Value</th></tr>"
        arr = self.__storage.getTagsOfSpecFileVersion(versionId)
        latesttag = ""
        latestvalue = ""
        diststring = ""
        lastdiststringline = ""
        for h in arr:
            if (h["name"] == latesttag) and (h["value"] == latestvalue):
                diststring = diststring + h["distroarchtag"] + " "
            else:
                if latesttag != "":
                    if lastdiststringline != diststring:
                        contents = contents + "<tr><td colspan=\"2\">Distroarch ids: " + diststring + "</td></tr>"
                        lastdiststringline = diststring
                    contents = contents + "<tr><td>" + latesttag + "</td>"
                    contents = contents + "<td>" + latestvalue + "</td></tr>\n"
                diststring = h["distroarchtag"]
                latesttag = h["name"]
                latestvalue = h["value"]
        if latesttag != "":
            if lastdiststringline != diststring:
                contents = contents + "<tr><td colspan=\"2\">Distroarch ids: " + diststring + "</td></tr>"
                lastdiststringline = diststring
            contents = contents + "<tr><td>" + latesttag + "</td>"
            contents = contents + "<td>" + latestvalue + "</td></tr>\n"
        contents = contents + "</table>"
        self.generateFullPage(contents,["Packages", shortName, "Version " + str(versionId) + " of " + shortName], "packages/" + shortName + "/" + shortName + "-version-" + str(versionId) + ".html", shortName + " spec file, version " + str(versionId))
        
        
    def generateOnePackagePageSpecPage(self,specRepoSpecFile):
        shortName = specRepoSpecFile.getSubDir()
        contents = ""
        contents = contents + "<a href=\"info.html\">Back to the " + shortName + " rpms</a><br />"
        contents = contents + "<h2>Current spec in pydar2:</h2><br />"
        contents = contents + "<pre>"
        contents = contents + self.__getFileContents(specRepoSpecFile.getFullPath())
        contents = contents + "</pre>"
        self.generateFullPage(contents, ["Packages",shortName, shortName + " spec file"], "packages/" + shortName  + "/" + shortName + "-spec.html", shortName + " spec file : Dries RPM Repository")
        htcontents = "RewriteEngine on\n" + 'RewriteRule ^.*spec$ ' + shortName + "-spec.html" + "\n"
        self.createFile("packages/" + shortName  + "/.htaccess", htcontents)
        
    def getDistroNameFromRpmName(self, rpm):
        for distro in self.__rpmlists:
            for suf in distro.suffixes:
                if rpm.rfind(suf) > 0:
                    return distro.fullName
        return "unknown"
        
    def generateOnePackagePageFirstPage(self, specRepoSpecFile, versions, distNameValueArr):
        shortName = specRepoSpecFile.getSubDir()
        contents = ""
        contents = contents + "<h2>General info:</h2>"
        
        # for now, just get any value
        name = ""
        summary = ""
        group = ""
        url = ""
        packager = ""
        vendor = ""
        description = ""
        version = ""
        release = ""
        epoch = ""
        authority = ""
        license = ""
        upstream = ""
        screenshot = ""
        for h in distNameValueArr:
            if h['name'] == "NAME":
                name = h['value']
            if h['name'] == "SUMMARY":
                summary = h['value']
            if h['name'] == "GROUP":
                group = h['value']
            if h['name'] == "URL":
                url = h['value']
            if h['name'] == "PACKAGER":
                packager = h['value']
            if h['name'] == "VENDOR":
                vendor = h['value']
            if h['name'] == "DESCRIPTION":
                description = h['value']
            if h['name'] == "VERSION":
                version = h['value']
            if h['name'] == "RELEASE":
                release = h['value']
            if h['name'] == "EPOCH":
                epoch = h['value']
            if h['name'] == "# Authority":
                authority = h['value']
            if h['name'] == "LICENSE":
                license = h['value']
            if h['name'] == "# Upstream":
                upstream = h['value']
            if h['name'] == "# Screenshot":
                screenshot = h['value']
            
        contents = contents + "Name: " + name + "<br />"
        contents = contents + "Summary: " + summary + "<br />"
        contents = contents + "Authority: " + authority + "<br />"
        contents = contents + "Upstream: " + upstream + "<br />"
        contents = contents + "Version: " + version + "<br />"
        contents = contents + "Release: " + release + "<br />"
        contents = contents + "Epoch: " + epoch + "<br />"
        contents = contents + "License: " + license + "<br />"
        contents = contents + "Group: " + group + "<br />"
        contents = contents + "URL: " + url + "<br />"
        contents = contents + "Description: <br /><pre>" + description + "</pre><br />"
        if screenshot != "":
            contents = contents + "Screenshot: <a href=\"" + screenshot + "\">" + screenshot + "</a><br />"
        
        
        
        contents = contents + "<h2>Rpms:</h2>"
        for f in specRepoSpecFile.rpmfilenames:
            #print f.rfind("/") , len(f)
            short = f[f.rfind("/") +1: len(f)]
            distroName = self.getDistroNameFromRpmName(short)
            contents = contents + "<a href=\"" + f + "\">" + short + "</a>&nbsp;&nbsp;&nbsp;" + distroName + "<br />"
        contents = contents + "<h2>Srpms:</h2>"
        for f in specRepoSpecFile.srpmfilenames:
            short = f[f.rfind("/")+1:  len(f)]
            distroName = self.getDistroNameFromRpmName(short)
            contents = contents + "<a href=\"" + f + "\">" + short + "</a>&nbsp;&nbsp;&nbsp;" + distroName + "<br />"
        contents = contents + "<h2>Spec info:</h2>"
        contents = contents + "Spec filename: " + specRepoSpecFile.getFileName() + "<br />"
        contents = contents + "View spec: <a href=\"" + specRepoSpecFile.getSubDir() + "-spec.html\">" + specRepoSpecFile.getSubDir() + " spec file</a><br />"
        svnUrl = "http://svn.rpmforge.net/svn/trunk/rpms/" + specRepoSpecFile.getSubDir() + "/" + specRepoSpecFile.getFileName() 
        contents = contents +  "Rpmforge Subversion URL: <a href=\"" + svnUrl + "\">" + svnUrl + "</a><br />"
        contents = contents + "<h2>Pydar2 info:</h2>"
        contents = contents + "Id in db: " + str(specRepoSpecFile.getId()) + "<br />"
        contents = contents + "Checksum: " + specRepoSpecFile.getCheckSum() + "<br />"
        contents = contents + "<h2>Versions in pydar2:</h2>"
        for versionId in versions:
            contents = contents  + "<a href=\"" + shortName + "-version-" + str(versionId) + ".html\">version " + str(versionId) + " of " + shortName + "</a><br />"
        self.generateFullPage(contents, ["Packages",shortName + " RPM"], "packages/" + shortName  + "/info.html", shortName + " RPM : Dries RPM Repository")
        
class TmpOptions:
    def __init__(self):
        self.masterconfig = None
    
c = Config.getInstance()
c.specifyGetOptOptions(TmpOptions())
c.getSpecRepositoryList().updateAllFileLists()
r = c.getSpecRepositoryList().getSpecRepositoryByName("rpmforge")
RpmforgeSite(r)
